from rdkit import Chem

try:
    from rdkit.Chem import rdFMCS as MCS
except ImportError:
    from rdkit.Chem import MCS
from rdkit.Chem import AllChem, Draw
from tqdm import tqdm
import os

SMARTS_PATTERN = "[*;R]-;!@[*]"

""" contribution from Hans de Winter """
from rdkit import Chem
from rdkit.Chem import AllChem


def _InitialiseNeutralisationReactions():
    patts = (
        # Imidazoles
        ("[n+;H]", "n"),
        # Amines
        ("[N+;!H0]", "N"),
        # Carboxylic acids and alcohols
        ("[$([O-]);!$([O-][#7])]", "O"),
        # Thiols
        ("[S-;X1]", "S"),
        # Sulfonamides
        ("[$([N-;X2]S(=O)=O)]", "N"),
        # Enamines
        ("[$([N-;X2][C,N]=C)]", "N"),
        # Tetrazoles
        ("[n-]", "[nH]"),
        # Sulfoxides
        ("[$([S-]=O)]", "S"),
        # Amides
        ("[$([N-]C=O)]", "N"),
    )
    return [(Chem.MolFromSmarts(x), Chem.MolFromSmiles(y, False)) for x, y in patts]


_reactions = None


def NeutraliseCharges(mol, reactions=None, as_mol=False):
    global _reactions
    smiles = Chem.MolToSmiles(mol, True)
    if reactions is None:
        if _reactions is None:
            _reactions = _InitialiseNeutralisationReactions()
        reactions = _reactions
    replaced = False
    for i, (reactant, product) in enumerate(reactions):
        while mol.HasSubstructMatch(reactant):
            replaced = True
            rms = AllChem.ReplaceSubstructs(mol, reactant, product)
            mol = rms[0]
    if replaced:
        if as_mol:
            return (mol, True)
        else:
            return (Chem.MolToSmiles(mol, True), True)
    else:
        if as_mol:
            return (mol, True)
        else:
            return (smiles, False)


def conv_at_xe(x):
    return NeutraliseCharges(Chem.MolFromSmiles(x.replace("[At]", "[Xe]")))[0]


def write_results(input_dict):
    """
    Write out a structure list of results.
    :param input_dict:
    :return:
    """
    out_imgs = {}
    for mol in input_dict:
        mols = []
        for new_mols in input_dict[mol]:
            m = Chem.MolFromSmiles(new_mols)
            mols.append(m)
        if len(mols) > 2:
            p = Chem.MolFromSmarts(MCS.FindMCS(mols).smarts)
            AllChem.Compute2DCoords(p)
            for m in mols:
                AllChem.GenerateDepictionMatching2DStructure(m, p)
        out_imgs[mol] = Draw.MolsToGridImage(mols, useSVG=True)
        # Write out the image
    return out_imgs


def get_frag_list(str_find, input_mol):
    """
    Get the list of fragments
    :param str_find:
    :param input_mol:
    :return:
    """
    return [
        x.replace(str_find, "Xe")
        for x in Chem.MolToSmiles(input_mol, isomericSmiles=True).split(".")
    ]


def get_comb_index(bi_1, bi_2):
    """

    :param bi_1:
    :param bi_2:
    :return:
    """
    return bi_1 + (100 * (bi_2 + 1))


def ret_comb_index(bi_tot, get_indices=False, isotope=None):
    """

    :param bi_tot:
    :return:
    """
    bi_1 = int(str(bi_tot)[-2:])
    bi_2 = int(str(bi_tot)[0:-2])
    if get_indices:
        return (bi_1, bi_2 - 1, isotope)
    else:
        return (bi_1, bi_2 - 1)


def get_fragments(input_mol, iso_labels=True, get_index_iso_map=False):
    """
    Find the frgments for a given molecule
    :param input_mol:
    :return:
    """
    index_isotope_map = {}
    atom_indices = input_mol.GetSubstructMatches(Chem.MolFromSmarts(SMARTS_PATTERN))
    if atom_indices and iso_labels:
        counter = 100
        labels = []
        bs = []
        for bi in atom_indices:
            b = input_mol.GetBondBetweenAtoms(bi[0], bi[1])
            if counter in index_isotope_map:
                index_isotope_map[counter].append(b.GetIdx())
            else:
                index_isotope_map[counter] = [b.GetIdx()]
            labels.append((counter, counter))
            bs.append(b.GetIdx())
            counter += 1
        input_mol = Chem.FragmentOnBonds(input_mol, bs, dummyLabels=labels)
    elif atom_indices:
        bs = []
        labels = []
        for bi in atom_indices:
            b = input_mol.GetBondBetweenAtoms(bi[0], bi[1])
            bs.append(b.GetIdx())
            comb_index = get_comb_index(bi[0], bi[1])
            labels.append((comb_index, comb_index))
        input_mol = Chem.FragmentOnBonds(input_mol, bs, dummyLabels=labels)
        return get_frag_list(str_find="*", input_mol=input_mol)
    if get_index_iso_map:
        return get_frag_list(str_find="*", input_mol=input_mol), index_isotope_map
    else:
        return get_frag_list(str_find="*", input_mol=input_mol)


def get_num_ring_atoms(input_mol):
    """
    Get the number of ring atoms
    :param input_mol:
    :return:
    """
    num_ring_atoms = 0
    split_index = 0
    split_indices = []
    for atom in input_mol.GetAtoms():
        if atom.IsInRing():
            num_ring_atoms += 1
        else:
            split_indices.append(split_index)
        split_index += 1
    return num_ring_atoms, split_indices


def simplified_graph(input_smiles, iso_flag=True):
    """
        The simplified graph representation for the edge uses the daylight
    function dt_molgraph to
    1) set all bond orders to one, 2) remove aromaticity, 3) set the hydrogen count,
    4) remove charges and set masses to zero. 5)  Additionally we set every ring atom element to carbon.
    This is Anthony Bradley's version of the algorithm.
        :param input_smiles: the input smiles to simplify
        :param iso_flag: a boolean to determine if we should include chirality etc
        :return:
    """
    mol = Chem.MolFromSmiles(input_smiles)
    for atom in mol.GetAtoms():
        atom.SetFormalCharge(0)
        if atom.IsInRing():
            atom.SetAtomicNum(6)
            atom.SetIsAromatic(False)
        for bond in atom.GetBonds():
            bond.SetBondType(Chem.BondType.SINGLE)
        if not iso_flag:
            atom.SetChiralTag(Chem.ChiralType.CHI_OTHER)
    return Chem.MolToSmiles(mol, isomericSmiles=True)


def recombine_edges(output_edges):
    """
    Recombine a list of edges based on their rules.
    Recombines identical Xe isotopes. Remove isotopes.
    :param output_edges:
    :return:
    """
    mol = Chem.MolFromSmiles(".".join(output_edges))
    # Dictionary of atom's to bond together and delete if they come in pairs
    iso_dict = {}
    for atom in mol.GetAtoms():
        if atom.GetAtomicNum() == 54:
            # Get the isotope
            iso = atom.GetIsotope()
            if iso in iso_dict:
                iso_dict[iso].append(get_info(atom))
            else:
                iso_dict[iso] = [get_info(atom)]
    mw = Chem.RWMol(mol)
    # Add bonds first
    del_indices = []
    for isotope in iso_dict:
        if len(iso_dict[isotope]) > 1:
            mw.AddBond(
                iso_dict[isotope][0][1], iso_dict[isotope][1][1], Chem.BondType.SINGLE
            )
            del_indices.append(iso_dict[isotope][0][0])
            del_indices.append(iso_dict[isotope][1][0])
    # Now delete atoms
    del_count = 0
    for atom_index in sorted(del_indices):
        mw.RemoveAtom(atom_index - del_count)
        del_count += 1
    Chem.SanitizeMol(mw)
    return Chem.MolToSmiles(mw, isomericSmiles=True)


def rebuild_smi(input_list, ring_ring):
    """
    Rebuild a SMILES
    :param input_list: the list of fragments to be rebuilt
    :param ring_ring: a boolean - indicating if this is a ring ring split
    :return:
    """
    if ring_ring:
        rebuilt_smi = ".".join(input_list)
    else:
        rebuilt_smi = recombine_edges(input_list)
    return rebuilt_smi


def make_child_mol(rebuilt_smi):
    """
    Make the child molecule
    :param rebuilt_smi:
    :return:
    """
    mol = Chem.MolFromSmiles(rebuilt_smi)
    # TODO proper warning messages for thsee two exceptions
    if mol is None:
        return None
    # TODO - check that non isomeric is ok here
    new_smi = Chem.MolToSmiles(mol, isomericSmiles=False).replace("[Xe]", "[H]")
    mol = Chem.MolFromSmiles(new_smi)
    # TODO proper warning messages for thsee two exceptions
    if mol is None:
        return None
    return Chem.CanonSmiles(new_smi)


def get_info(atom):
    """
    Get the needed info for an atom
    :param atom:
    :return:
    """
    return [atom.GetIdx(), atom.GetNeighbors()[0].GetIdx()]


def get_type(smiles):
    """
    Get the Type of a given entity
    :param smiles: the input SMILES
    :return: the type (FG or RING)
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[*;R]")):
        return "RING"
    return "FG"


def get_driver(url="neo4j", neo4j_auth="neo4j/neo4j"):
    """
    Get the driver to the network connection using the bolt service
    at the URI provided. If an authentication string is provided
    (i.e. a typical NEO4J_AUTH value of <username>/<password>) then
    authentication is assumed.
    :return: the driver for the graph database
    """
    from neo4j import GraphDatabase

    auth_parts = neo4j_auth.split("/")
    if len(auth_parts) == 2:
        driver = GraphDatabase.driver(
            "bolt://" + url + ":7687", auth=(auth_parts[0], auth_parts[1])
        )
    else:
        driver = GraphDatabase.driver("bolt://" + url + ":7687")

    return driver


def get_ring_ring_splits(input_mol, labels=False, do_comb_index=False):
    """
    Get and break Atom-Atom pairs in two different rings.
    :param input_mol:
    :return:
    """
    # TODO Fix for fused e.g.s
    RI = input_mol.GetRingInfo()
    rings = RI.AtomRings()
    out_mols = []
    bonds = [item for sublist in RI.BondRings() for item in sublist]
    bs = []
    for bond in input_mol.GetBonds():
        if bond.GetIdx() in bonds:
            continue
        id_one = bond.GetBeginAtomIdx()
        id_two = bond.GetEndAtomIdx()
        # Now find all pairs that are in both
        for ring in rings:
            if id_one in ring:
                for ring_two in rings:
                    if ring == ring_two:
                        continue
                    if id_two in ring_two:
                        bs.append(bond)
    if bs:
        for b in bs:
            if labels:
                nm = Chem.FragmentOnBonds(
                    input_mol,
                    [b.GetIdx()],
                    dummyLabels=[(b.GetBeginAtomIdx(), b.GetEndAtomIdx())],
                )
                mols = [
                    x.replace("*", "Xe")
                    for x in Chem.MolToSmiles(nm, isomericSmiles=True).split(".")
                ]
            elif do_comb_index:
                print(b)

                comb_index = get_comb_index(b.GetBeginAtomIdx(), b.GetEndAtomIdx())
                nm = Chem.FragmentOnBonds(
                    input_mol, [b.GetIdx()], dummyLabels=[(comb_index, comb_index)]
                )
                mols = [
                    x.replace("*", "Xe")
                    for x in Chem.MolToSmiles(nm, isomericSmiles=True).split(".")
                ]
            else:
                nm = Chem.FragmentOnBonds(input_mol, [b.GetIdx()], dummyLabels=[(1, 1)])
                # Only takes first
                mols = [
                    x.replace("1*", "Xe")
                    for x in Chem.MolToSmiles(nm, isomericSmiles=True).split(".")
                ]
            out_mols.append(mols)
        return out_mols


def add_child_and_edge(
    new_list, input_node, excluded_smi, node_holder, ring_ring=False
):
    """
    :param input_pair:
    :return:
    """
    # Rebuild the molecule
    rebuilt_smi = rebuild_smi(new_list, ring_ring)
    # Turn into child molecule
    child_smi = make_child_mol(rebuilt_smi)
    if child_smi is None:
        return
    # Now generate the edges with input and this node
    new_node, is_new = node_holder.create_or_retrieve_node(child_smi)
    node_holder.create_or_retrieve_edge(excluded_smi, rebuilt_smi, input_node, new_node)
    # Now generate the children for this too
    if is_new:
        create_children(new_node, node_holder)


def canon_input(smi, isomericSmiles=True):
    # Decharge in this step too
    iso_smiles = NeutraliseCharges(Chem.MolFromSmiles(str(smi)))[0]
    if not isomericSmiles:
        # need to add isomericSmiles=False back in...
        return Chem.MolToSmiles(
            Chem.MolFromSmiles(iso_smiles)
        )  # , isomericSmiles=False)
    else:
        return iso_smiles


def create_children(input_node, node_holder):
    """
    Create a series of edges from an input molecule. Iteratively
    :param input_node:
    :return:
    """
    # Get all ring-ring splits
    ring_ring_splits = get_ring_ring_splits(input_node.RDMol)
    if ring_ring_splits:
        for ring_ring_split in ring_ring_splits:
            add_child_and_edge(
                ring_ring_split, input_node, "[Xe]", node_holder, ring_ring=True
            )
    fragments = get_fragments(input_node.RDMol)
    if len(fragments) < 2:
        return
    # Now remove one item on each iteration
    for i in range(len(fragments)):
        new_list = []
        for j, item in enumerate(fragments):
            if i == j:
                excluded_smi = item
                continue
            new_list.append(item)
        add_child_and_edge(new_list, input_node, excluded_smi, node_holder)


def neutralise_3d_mol(input_mol):
    neutral_mol = NeutraliseCharges(Chem.MolFromMolBlock(input_mol), as_mol=True)[0]
    return AllChem.ConstrainedEmbed(neutral_mol, Chem.MolFromMolBlock(input_mol))


def write_data(output_dir, node_holder, attrs):
    out_f = open(os.path.join(output_dir, "nodes.txt"), "w")
    for node in node_holder.node_list:
        out_f.write(str(node))
        out_f.write("\n")
    out_f = open(os.path.join(output_dir, "edges.txt"), "w")
    for edge in node_holder.get_edges():
        out_f.write(str(edge))
        out_f.write("\n")
    out_f = open(os.path.join(output_dir, "attributes.txt"), "w")
    for attr in attrs:
        out_f.write(str(attr))
        out_f.write("\n")


def build_network(attrs, node_holder):
    # Create the nodes and test with output
    for attr in tqdm(attrs):
        node, is_node = node_holder.create_or_retrieve_node(attr.SMILES)
        if is_node:
            create_children(node, node_holder)
    return node_holder


def add_node(tx, smiles, hac, chac, osmiles):
    tx.run(
        "MERGE (:F2 { smiles: $smiles, hac: toInt($hac), chac: toInt($chac), osmiles: $osmiles})",
        smiles=smiles,
        hac=hac,
        chac=chac,
        osmiles=osmiles,
    )


def add_edge(tx, smiles, smiles_two, edge_meta):
    tx.run(
        "MATCH (n1:F2 { smiles: $smiles}), (n2:F2 { smiles: $smiles_two}) MERGE (n1)-[:FRAG{label:$edge_meta}]->(n2)",
        smiles=smiles,
        smiles_two=smiles_two,
        edge_meta=edge_meta,
    )


def add_attr(tx, smiles, attr):
    tx.run(
        "MATCH (n:F2 { smiles: $smiles} ) set n:MOL, n:Mol, n.Mol=$attr",
        smiles=smiles,
        attr=attr,
    )



from rdkit import Chem
from rdkit.Geometry.rdGeometry import Point3D
from rdkit.Chem import AllChem, RWMol, Atom
from frag.utils.network_utils import (
    rebuild_smi,
    get_ring_ring_splits,
    get_fragments,
    ret_comb_index,
    conv_at_xe,
)
import re

XE_PATT = r"[0-9]{3,}Xe"


def get_mol(mol_parser, mol_data):
    rd_mol = mol_parser(mol_data)
    return RWMol(AllChem.AddHs(rd_mol))


def decorate_smi(input_smi):
    """
    Decorate an input SMILES with a pseudeo molecule with all desirable changes.
    This can be added to the list of input molecules - and then we know if we get an Antimony back - it's not a real trans.
    :param input_smi: the input smiles
    :return: a list of SMILES around a given molecule.
    """
    # Add replacement groups (e.g. At) to all Ring positions in turn.
    return decorate_mol(get_mol, Chem.MolFromSmiles, input_smi)


def decorate_3d_mol(input_mol_file, get_indices=False):
    res_dict = decorate_mol(
        get_mol, Chem.MolFromMolBlock, input_mol_file, three_d_mol=True
    )
    out_dict = {}
    for res in res_dict:
        atom_pairs = res_dict[res]
        mol = AllChem.AddHs(Chem.MolFromMolBlock(input_mol_file), addCoords=True)
        conf = mol.GetConformer()
        for i, atom_pair in enumerate(atom_pairs):
            if get_indices:
                new_atom_pair = (atom_pair[0], atom_pair[1], "NA", True)
                out_dict[conv_at_xe(res) + "__" + str(i)] = [new_atom_pair]
            else:
                atom_one = conf.GetAtomPosition(atom_pair[0])
                atom_two = conf.GetAtomPosition(atom_pair[1])
                out_dict[conv_at_xe(res) + "__" + str(i)] = [
                    (atom_one.x, atom_one.y, atom_one.z),
                    (atom_two.x, atom_two.y, atom_two.z),
                ]
    return out_dict


def decorate_mol(get_new_mol, mol_parse, mol_data, three_d_mol=False):
    mol = get_new_mol(mol_parse, mol_data)
    patt = Chem.MolFromSmarts("[*;R]-;!@[H]")
    # Get the list of atom Indices to replace
    out_atom_repls = [x for x in mol.GetSubstructMatches(patt)]
    # Now replace with At - an produce a new mol everytime
    new_mols = {}
    for atom_pairs in out_atom_repls:
        atom = atom_pairs[1]
        rw_mol = get_new_mol(mol_parse, mol_data)
        rw_mol.ReplaceAtom(atom, Atom(85))
        newer_mol = rw_mol.GetMol()
        this_mol = Chem.MolToSmiles(
            Chem.MolFromSmiles(Chem.MolToSmiles(newer_mol, isomericSmiles=True)),
            isomericSmiles=True,
        )
        if three_d_mol:
            if this_mol in new_mols:
                new_mols[this_mol].append(atom_pairs)
            else:
                new_mols[this_mol] = [atom_pairs]
        else:
            new_mols[this_mol] = atom_pairs
    return new_mols


def deletion_linker_smi(input_smi, iso_labels=True):
    mol = Chem.MolFromSmiles(input_smi)
    return deletion_linker_mol(mol, iso_labels)


def deletion_linker_sd(input_mol, iso_labels=True):
    mol = Chem.MolFromMolBlock(input_mol)
    return deletion_linker_mol(mol, iso_labels)


def get_3d_vects_for_mol(input_mol, iso_labels=True):
    tot_dict = del_link_coord(input_mol, iso_labels=iso_labels)
    tot_dict["additions"] = decorate_3d_mol(input_mol)
    return tot_dict


def get_vect_indices_for_mol(input_mol):
    mol = Chem.MolFromSmiles(Chem.MolToSmiles(Chem.MolFromMolBlock(input_mol)))
    AllChem.EmbedMolecule(mol)
    input_mol = Chem.MolToMolBlock(mol)
    tot_dict = del_link_coord(input_mol, get_indices=True, iso_labels=False)
    tot_dict["additions"] = decorate_3d_mol(input_mol, get_indices=True)
    return tot_dict


def convert_dict_to_mols(tot_dict):
    """
    :param tot_dict:
    :return:
    """
    mol_list = []
    for smiles in tot_dict:
        # Now generate the molecules for that
        mol = RWMol()
        atoms = tot_dict[smiles]
        print(atoms)
        for atom in atoms:
            atom = Atom(6)
            mol.AddAtom(atom)
        # for i in range(len(atoms)-1):
        #     mol.AddBond(i,i+1)
        mol = mol.GetMol()
        AllChem.EmbedMolecule(mol)
        conf = mol.GetConformer()
        for i, atom in enumerate(atoms):
            point_3d = Point3D(atom[0], atom[1], atom[2])
            conf.SetAtomPosition(i, point_3d)
        mol = conf.GetOwningMol()
        mol.SetProp("_Name", smiles)
        mol_list.append(mol)
    return mol_list


def del_link_coord(input_mol, get_indices=False, iso_labels=False):
    tot_vals = deletion_linker_sd(input_mol, iso_labels=iso_labels)
    deletions = [Chem.MolToSmiles(x, isomericSmiles=True) for x in tot_vals[0]]
    linkers = [Chem.MolToSmiles(x, isomericSmiles=True) for x in tot_vals[1]]
    ring = [Chem.MolToSmiles(x, isomericSmiles=True) for x in tot_vals[2]]

    if not iso_labels:
        iso_vals = deletion_linker_sd(input_mol, iso_labels=True)
    else:
        iso_vals = tot_vals
    deletions_iso = [Chem.MolToSmiles(x, isomericSmiles=True) for x in iso_vals[0]]
    linkers_iso = [Chem.MolToSmiles(x, isomericSmiles=True) for x in iso_vals[1]]
    ring_iso = [Chem.MolToSmiles(x, isomericSmiles=True) for x in iso_vals[2]]

    out_d = {"linkers": {}, "deletions": {}, "ring": {}}
    for i, x in enumerate(linkers):
        ret_ans = get_atom_coords(
            x, Chem.MolFromMolBlock(input_mol), get_indices, linkers_iso[i]
        )
        out_d["linkers"][ret_ans[0]] = ret_ans[1:]
    for i, x in enumerate(deletions):
        ret_ans = get_atom_coords(
            x, Chem.MolFromMolBlock(input_mol), get_indices, deletions_iso[i]
        )
        out_d["deletions"][ret_ans[0]] = ret_ans[1:]
    for i, x in enumerate(ring):
        ret_ans = get_atom_coords(
            x, Chem.MolFromMolBlock(input_mol), get_indices, ring_iso[i]
        )
        out_d["ring"][ret_ans[0]] = ret_ans[1:]
    return out_d


def deletion_linker_mol(mol, iso_labels=True):
    """
    Produce all the linker deletion SMILES and replace with Li
    :param input_smi: the input SMI
    :return:
    """
    nr = mol.GetRingInfo().NumRings()
    fragments = get_fragments(mol, iso_labels=iso_labels)
    out_mols = []
    linker_mol_list = []
    ring_repl_list = []
    ring_ring_splits = get_ring_ring_splits(mol, do_comb_index=True)
    if ring_ring_splits:
        for ring_ring_split in ring_ring_splits:
            rebuilt_smi = rebuild_smi(ring_ring_split, ring_ring=True)
            new_mol = Chem.MolFromSmiles(rebuilt_smi)
            if new_mol.GetRingInfo().NumRings() < nr:
                continue
            linker_mol_list.append(new_mol)
    for i in range(len(fragments)):
        new_list = []
        for j, item in enumerate(fragments):
            if i == j:
                continue
            new_list.append(item)
        rebuilt_smi = rebuild_smi(new_list, ring_ring=False)
        if "." in rebuilt_smi:
            new_mol = Chem.MolFromSmiles(rebuilt_smi)
            # Only consider linker replacements
            if new_mol.GetRingInfo().NumRings() < nr:
                ring_repl_list.append(new_mol)
            else:
                linker_mol_list.append(new_mol)
            continue
        new_mol = Chem.MolFromSmiles(rebuilt_smi)
        # If the resulting
        if new_mol.GetRingInfo().NumRings() < nr:
            continue
        out_mols.append(new_mol)
    return out_mols, linker_mol_list, ring_repl_list


def find_atom_pairs(smiles_input, get_indices, iso_smiles):
    """
    Find the indices of the atom pairs from a SMILES input
    :param smiles_input:
    :return:
    """
    matches = re.findall(XE_PATT, smiles_input)
    if iso_smiles:
        iso_matches = re.findall(XE_PATT, iso_smiles)
    else:
        iso_matches = None
        isotope = None
    ind_list = []
    for i, match in enumerate(matches):
        index = int(match[:-2])
        if iso_matches:
            isotope = int(iso_matches[i][:-2])
        indices = ret_comb_index(index, get_indices, isotope)
        ind_list.append(indices)
    return ind_list


def get_atom_coords(smiles_input, mol, get_indices=False, iso_smiles=None):
    """
    Get a
    :param smiles_input:
    :param mol:
    :return:
    """
    ind_list = find_atom_pairs(smiles_input, get_indices, iso_smiles)
    clean_smi = re.sub(XE_PATT, "Xe", smiles_input)
    out_list = [clean_smi]
    conf = mol.GetConformer()
    for atom_pair in ind_list:
        atom_one = conf.GetAtomPosition(atom_pair[0])
        atom_two = conf.GetAtomPosition(atom_pair[1])
        if get_indices:
            out_list.append((atom_pair[0], atom_pair[1], atom_pair[2], False))
        else:
            out_list.extend(
                [
                    (atom_one.x, atom_one.y, atom_one.z),
                    (atom_two.x, atom_two.y, atom_two.z),
                ]
            )
    return out_list


def link_li(rebuilt_smi):
    mol = Chem.MolFromSmiles(rebuilt_smi)
    mol = RWMol(mol)
    bons = [x[0] for x in mol.GetSubstructMatches(Chem.MolFromSmarts("[Xe]"))]
    mol.AddBond(bons[0], bons[1])
    return mol.GetMol()


def addition_smi(input_smi):
    smis = decorate_smi(input_smi)
    return [Chem.MolFromSmiles(x) for x in smis]


def get_ring_removals(smi):
    rw_mol = RWMol(Chem.MolFromSmiles(smi))
    rings = rw_mol.GetRingInfo().AtomRings()
    out_mols = {}
    for ring in rings:
        new_mol = Chem.MolFromSmiles(smi)
        for atom in ring:
            new_mol.GetAtomWithIdx(atom).SetAtomicNum(0)
        Chem.DeleteSubstructs(new_mol, Chem.MolFromSmarts("[#0]"))
        Chem.GetMolFrags(new_mol)
        out_mols[Chem.MolToSmiles(new_mol, isomericSmiles=True)] = ring
    return out_mols


def conv_smiles(additions, deletions, linkers, ring_removals):
    additions = [conv_at_xe(Chem.MolToSmiles(x)) for x in additions]
    deletions = [Chem.MolToSmiles(x) for x in deletions]
    linkers = [Chem.MolToSmiles(x) for x in linkers]
    ring_removals = [Chem.MolToSmiles(x) for x in ring_removals]
    linkers.extend(ring_removals)
    return additions, deletions, linkers, ring_removals


def get_add_del_link(smi, asSmiles=True):
    additions = addition_smi(smi)
    res = deletion_linker_smi(smi)
    linkers = res[1]
    ring_removals = res[2]
    deletions = res[0]
    if asSmiles:
        additions, deletions, linkers, ring_removals = conv_smiles(
            additions, deletions, linkers, ring_removals
        )
    return [additions, deletions, linkers]



from frag.network.decorate import decorate_smi
import argparse
from rdkit import Chem
from rdkit.Chem.Scaffolds import MurckoScaffold
from tqdm import tqdm
from frag.network.models import Attr

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Decorate a library of molecules for insertion to the database."
    )
    parser.add_argument("--input_smi")
    parser.add_argument("--output_attr")
    args = parser.parse_args()
    out_smi = open(args.output_attr, "w")
    for mol in tqdm(
        Chem.SmilesMolSupplier(
            args.input_smi, delimiter=",", smilesColumn=1, nameColumn=0
        )
    ):
        this_smi = Chem.MolToSmiles(mol, isomericSmiles=True)
        new_smis = decorate_smi(this_smi)
        new_murck = decorate_smi(MurckoScaffold.MurckoScaffoldSmiles(this_smi))
        # mol_frags = get_fragments(Chem.MolFromSmiles(this_smi),iso_labels=False)
        # new_smis.extend([x.replace("Xe","At") for x in mol_frags])
        new_smis.extend(new_murck)
        new_smis = list(set(new_smis))
        # Do this on original and on Murcko Scaffold
        name = mol.GetProp("_Name")
        new_attr = Attr(this_smi, ["EM", name])
        out_smi.write(str(new_attr) + "\n")
        for i, smi in enumerate(new_smis):
            new_attr = Attr(smi, ["EM", name + "_" + str(i)])
            out_smi.write(str(new_attr) + "\n")



import argparse
import os

from rdkit import Chem

from frag.network.models import NodeHolder, Attr
from frag.utils.network_utils import build_network, write_data
from frag.utils.parser import parse_mols

from tqdm import tqdm


def main():
    # Read in a SD or SMILES file - then write out into a specified directory
    parser = argparse.ArgumentParser(
        description="Convert a SMILES or SDFile to input for Astex Fragment network."
    )
    parser.add_argument("--input")
    parser.add_argument("--input_format", default="smi")
    parser.add_argument("--base_dir")
    parser.add_argument("--isomeric", dest="iso_flag", action="store_true")
    parser.add_argument("--non_isomeric", dest="iso_flag", action="store_false")
    parser.set_defaults(iso_flag=True)

    args = parser.parse_args()
    attrs = []
    id = 0
    mols = parse_mols(args.input, args.input_format)
    for x in tqdm(mols):
        if x is None:
            continue
        attr = Attr(
            Chem.CanonSmiles(Chem.MolToSmiles(x, isomericSmiles=True)),
            ["EM", x.GetProp("_Name")],
        )
        attrs.append(attr)
        id += 1
    if not os.path.isdir(args.base_dir):
        os.mkdir(args.base_dir)
    # Build the network
    node_holder = NodeHolder(iso_flag=args.iso_flag)
    node_holder = build_network(attrs, node_holder)
    # Write the data out
    write_data(args.base_dir, node_holder, attrs)


if __name__ == "__main__":
    main()



from neomodel import (
    StructuredNode,
    StructuredEdge,
    StringProperty,
    FloatProperty,
    IntegerProperty,
    RelationshipTo,
    RelationshipFrom,
    BooleanProperty,
)


class F2Edge(StructuredEdge):
    exclude_smiles = StringProperty()
    exclude_type = StringProperty()
    rebuilt_smiles = StringProperty()
    rebuilt_ring_smiles = StringProperty()
    rebuilt_type = StringProperty()
    excluded_ring_smiles = StringProperty()
    # The Exclude and Rebuilt smiles combined
    both_sides = StringProperty(unique_index=True)


class BaseMolecule(StructuredNode):
    smiles = StringProperty(unique_index=True)
    heavy_atom_count = IntegerProperty()
    ring_atom_count = IntegerProperty()
    ring_smiles = StringProperty()
    # Annotations of price and molecule id
    price = IntegerProperty()
    mol_id = StringProperty()


class Compound(BaseMolecule):
    """
    A molecule - a real purchasable molecule
    """

    fragments = RelationshipTo("Fragment", "FRAG")
    conformers = RelationshipTo("Conformer", "CONFEDGE")
    parent_molecules = RelationshipTo("Compound", "PARENTEDGE")


class Fragment(BaseMolecule):
    """
    A fragment - not a real molecule
    """

    molecules = RelationshipFrom(Compound, "FRAG", model=F2Edge)


class Conformer(StructuredNode):
    """
    The 3D conformer of a molecule - either Docked or imaginary
    """

    # Going to go with uuid - seems as good as anything - this connects to the Django database for 3D information
    unique_string = StringProperty(unique_index=True)
    conformer = RelationshipFrom(Compound, "CONFEDGE")
    score = FloatProperty()
    is_xtal_pose = BooleanProperty()
    threed_sim_to_reference = FloatProperty()
    rmsd_to_xtal_pose = FloatProperty()
    # Connect to Molecule and Protein


class ConformerFeature(StructuredNode):
    """
    A pharmacophore feature for a given Conformer
    """

    # Unique id - uuid:PHARMAFEATURE:INDEX
    unique_string = StringProperty(unique_index=True)
    pharma_feature = StringProperty()
    index = IntegerProperty()
    # Connect to SpecificResidue and to other SpecificResidueFeature annotated by distance and angle


class Target(StructuredNode):
    """
    A target is defined as a unique Uniprot accessiion
    """

    # A given uniprot accession
    uniprot_id = StringProperty(unique_index=True)
    # Now a series of properties
    sequence = StringProperty()
    # Connect Target-Target (sequence similarity data from Biojava)


class Protein(StructuredNode):
    """
    A unique set of 3D coordinates for a protein structure
    """

    # A given protein structure
    sequence = StringProperty()
    # Connect Protein-Target and Protein-Protein annotate by they're sequence identity


class GeneralResidue(StructuredNode):
    """
    A unique residue type - as defined by PDB RES codes
    """

    # A particular type of residue (defined by PDB RES code)
    res_code = StringProperty(unique_index=True)
    # Connect Residue to Residue based on their similarity. Human defined at first


class SpecificResidue(StructuredNode):
    """
    A specific 3D coords for a given residue
    """

    # Unique id - PDBID:CHAINID:RES_NUM:ALTLOC
    unique_string = StringProperty(unique_index=True)
    pdb_id = StringProperty()
    chain_id = StringProperty()
    residue_number = StringProperty()
    res_id = StringProperty()
    # Connect SpecificResidue to GeneralResidue and Protein
    # and SpecificResidue to SpecificResidue based on orientation and distance


class SpecificResidueFeature(StructuredNode):
    """
    A pharmacophore feature for a given SpecificResidue
    """

    # Unique id - PDBID:CHAINID:RES_NUM:ALTLOC:PHARMATYPE:INDEX
    unique_string = StringProperty(unique_index=True)
    # The 3 Letter code residue id
    res_id = StringProperty()
    pharma_type = StringProperty()
    index = IntegerProperty()
    # Connect to SpecificResidue and
    # to other SpecificResidueFeature/ConformerFeature annotated by distance and angle



import pickle, time
from rdkit.Chem import rdSubstructLibrary
from rdkit import Chem
import tqdm


def generate_fps(input_smi):
    # start by building the fingerprint cache
    t1 = time.time()
    with open(input_smi, "r") as inf:
        ls = [[x.split()[0], x.split()[1]] for x in inf]
        ls.pop(0)
        with open(input_smi.replace(".smi", ".pkl"), "wb+") as pklf:
            for i, (smi, chemblid) in enumerate(ls):
                m = Chem.MolFromSmiles(smi, sanitize=False)
                fp = Chem.PatternFingerprint(m)
                pickle.dump(fp, pklf)
                if not (i + 1) % 10000:
                    print("Done", i + 1)
    t2 = time.time()
    print("That took %.2f seconds. The output has %d molecules." % (t2 - t1, i))


def read_in_lib(input_smi):
    t1 = time.time()
    mols = rdSubstructLibrary.CachedTrustedSmilesMolHolder()
    fps = rdSubstructLibrary.PatternHolder()
    with open(input_smi, "r") as inf:
        ls = [x.split() for x in inf]
        ls.pop(0)
        with open(input_smi.replace(".smi", ".pkl"), "rb") as pklf:
            for l in tqdm.tqdm(ls):
                smi = l[1]
                mols.AddSmiles(smi)
                fp = pickle.load(pklf)
                fps.AddFingerprint(fp)
    library = rdSubstructLibrary.SubstructLibrary(mols, fps)
    t2 = time.time()
    print(
        "That took %.2f seconds. The library has %d molecules."
        % (t2 - t1, len(library))
    )
    return library



import os
import numpy as np
from rdkit import Chem
from rdkit.Chem import AllChem, rdShapeHelpers
from rdkit.Chem.FeatMaps import FeatMaps
from rdkit import RDConfig


fdef = AllChem.BuildFeatureFactory(
    os.path.join(RDConfig.RDDataDir, "BaseFeatures.fdef")
)

fmParams = {}
for k in fdef.GetFeatureFamilies():
    fparams = FeatMaps.FeatMapParams()
    fmParams[k] = fparams

keep = (
    "Donor",
    "Acceptor",
    "NegIonizable",
    "PosIonizable",
    "ZnBinder",
    "Aromatic",
    "Hydrophobe",
    "LumpedHydrophobe",
)


def get_FeatureMapScore(small_m, large_m, score_mode=FeatMaps.FeatMapScoreMode.All):
    featLists = []
    for m in [small_m, large_m]:
        rawFeats = fdef.GetFeaturesForMol(m)
        # filter that list down to only include the ones we're intereted in
        featLists.append([f for f in rawFeats if f.GetFamily() in keep])
    fms = [
        FeatMaps.FeatMap(feats=x, weights=[1] * len(x), params=fmParams)
        for x in featLists
    ]
    fms[0].scoreMode = score_mode
    fm_score = fms[0].ScoreFeats(featLists[1]) / min(
        fms[0].GetNumFeatures(), len(featLists[1])
    )
    return fm_score


def score(reflig, prb_mols, ids, score_mode=FeatMaps.FeatMapScoreMode.All, p=False):
    ref = Chem.AddHs(reflig)
    idx = 0

    results_sucos = {}
    results_tani = {}

    smi_mol = Chem.MolToSmiles(prb_mols)

    for i in ids:
        prb = Chem.AddHs(Chem.MolFromMolBlock(Chem.MolToMolBlock(prb_mols, confId=i)))

        fm_score = get_FeatureMapScore(ref, prb, score_mode)
        fm_score = np.clip(fm_score, 0, 1)

        protrude_dist = rdShapeHelpers.ShapeProtrudeDist(
            ref, prb, allowReordering=False
        )
        protrude_dist = np.clip(protrude_dist, 0, 1)

        SuCOS_score = 0.5 * fm_score + 0.5 * (1 - protrude_dist)
        tanimoto_score = Chem.rdShapeHelpers.ShapeTanimotoDist(ref, prb)

        results_sucos[str(idx)] = SuCOS_score
        results_tani[str(idx)] = tanimoto_score

        if p:
            print("********************************")
            print("index: " + str(idx))
            print("SuCOS score:\t%f" % SuCOS_score)
            print("Tani score:\t%f" % tanimoto_score)
            print("********************************")

        idx += 1

    return results_sucos


def get_best_align(hit_mblock, elab_smiles):
    hit_mol = Chem.MolFromMolBlock(hit_mblock)
    elab_mol = Chem.MolFromSmiles(elab_smiles)
    ids = AllChem.EmbedMultipleConfs(elab_mol, numConfs=100, params=AllChem.ETKDG())

    for cid in ids:
        o3d = Chem.rdMolAlign.GetO3A(prbMol=elab_mol, refMol=hit_mol, prbCid=cid)
        o3d.Align()

    results_sucos = score(hit_mol, elab_mol, ids)
    best_i = list(results_sucos.values()).index(max(results_sucos.values()))
    elab_molblock = Chem.MolToMolBlock(elab_mol, confId=best_i)

    return elab_molblock


def get_core_mol(three_d_mol, core_mol):
    repl_sidechains = AllChem.ReplaceSidechains(three_d_mol, core_mol)
    if repl_sidechains:
        return AllChem.DeleteSubstructs(repl_sidechains, Chem.MolFromSmiles("*"))


def gen_conf_from_vector(input_mol_block, elaborated_smiles):
    # Get the mol
    m = get_best_align(input_mol_block, elaborated_smiles)
    return m


def generate_confs_for_vector(input_smiles, input_mol_block):
    confs = []
    for elaborated_smiles in input_smiles:
        confs.append(gen_conf_from_vector(input_mol_block, elaborated_smiles))
    return confs


def generate_confs_for_vectors(input_vectors, input_mol_block):
    confs = {}
    for vector in input_vectors:
        confs[vector] = generate_confs_for_vector(
            input_vectors[vector], input_mol_block
        )
    return confs


if __name__ == "__main__":
    conf = generate_confs_for_vectors(
        {"O=C(Nc1ccc([Xe])nc1)N1CCNCC1_1_ADDITION": ["COc1ccc(NC(=O)N2CCNCC2)cn1"]},
        "\n     RDKit          3D\n\n 15 16  0  0  0  0  0  0  0  0999 V2000\n   -8.0100   -6.1770   18.7960 O   0  0  0  0  0  0  0  0  0  0  0  0\n   -7.8350   -7.3610   19.1060 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -6.7970   -8.0980   18.5710 N   0  0  0  0  0  0  0  0  0  0  0  0\n   -6.0500   -7.8150   17.3890 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -6.2220   -6.6910   16.5800 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -5.5070   -6.4830   15.4790 N   0  0  0  0  0  0  0  0  0  0  0  0\n   -4.5740   -7.3730   15.1470 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -4.3180   -8.4870   15.8900 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -5.0700   -8.7240   17.0200 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -8.6530   -7.9840   20.0040 N   0  0  0  0  0  0  0  0  0  0  0  0\n   -8.8660   -9.4290   20.1410 C   0  0  0  0  0  0  0  0  0  0  0  0\n  -10.3240   -9.8000   19.9120 C   0  0  0  0  0  0  0  0  0  0  0  0\n  -11.2290   -8.6780   20.2340 N   0  0  0  0  0  0  0  0  0  0  0  0\n  -10.7100   -7.8960   21.3670 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -9.4210   -7.2020   20.9820 C   0  0  0  0  0  0  0  0  0  0  0  0\n  1  2  2  0\n  2  3  1  0\n  2 10  1  0\n  3  4  1  0\n  4  5  2  0\n  4  9  1  0\n  5  6  1  0\n  6  7  2  0\n  7  8  1  0\n  8  9  2  0\n 10 11  1  0\n 10 15  1  0\n 11 12  1  0\n 12 13  1  0\n 13 14  1  0\n 14 15  1  0\nM  END\n",
    )

    print(conf)



from rdkit import Chem
from frag.utils.network_utils import get_num_ring_atoms, get_type, simplified_graph


class NodeHolder(object):
    """
    A Class to hold a series of nodes
    """

    def __init__(self, iso_flag=True):
        self.node_list = set()
        self.edge_list = set()
        self.iso_flag = iso_flag

    def create_or_retrieve_node(self, child_smi):
        new_node = Node(Chem.MolFromSmiles(child_smi), self.iso_flag)
        if new_node not in self.node_list:
            self.node_list.add(new_node)
            return new_node, True
        return new_node, False

    def create_or_retrieve_edge(self, excluded_smi, child_smi, input_node, new_node):
        """
        :param excluded_smi:
        :param child_smi:
        :param input_node:
        :param new_node:
        :return:
        """
        new_edge = Edge(excluded_smi, child_smi, input_node, new_node, self.iso_flag)
        new_edge.NODES = [input_node, new_node]
        self.edge_list.add(new_edge)
        return new_edge

    def get_edges(self):
        """
        :return:
        """
        return set(self.edge_list)


class Node(object):
    """
    A Class to hold a Node on the graph
    """

    def __eq__(self, other):
        return self.HASH == other.HASH

    def __ne__(self, other):
        return self.HASH != other.HASH

    def __hash__(self):
        return self.HASH

    def __init__(self, input_mol=None, iso_flag=True):
        self.iso_flag = iso_flag
        if not input_mol:
            return
        if type(input_mol) == str:
            input_mol = Chem.MolFromSmiles(input_mol)
        self.SMILES = Chem.MolToSmiles(input_mol, isomericSmiles=self.iso_flag)
        self.HAC = input_mol.GetNumHeavyAtoms()
        self.RAC, split_indices = get_num_ring_atoms(input_mol)
        self.RING_SMILES = simplified_graph(self.SMILES, iso_flag=self.iso_flag)
        self.RDMol = input_mol
        self.EDGES = []
        self.HASH = hash(self.SMILES)

    def __str__(self):
        return " ".join(
            ["NODE", self.SMILES, str(self.HAC), str(self.RAC), self.RING_SMILES]
        )


class Edge(object):
    """
    A Class to hold an edge
    """

    def __eq__(self, other):
        return self.HASH == other.HASH

    def __ne__(self, other):
        return self.HASH != other.HASH

    def __hash__(self):
        return self.HASH

    def __init__(self, excluded_smi, rebuilt_smi, node_one, node_two, iso_flag=True):
        self.EXCLUDE_SMILES = Chem.MolToSmiles(
            Chem.MolFromSmiles(excluded_smi), isomericSmiles=False
        )
        self.EXCLUDE_TYPE = get_type(excluded_smi)
        self.REBUILT_SMILES = Chem.MolToSmiles(
            Chem.MolFromSmiles(rebuilt_smi), isomericSmiles=False
        )
        self.REBUILT_RING_SMILES = simplified_graph(rebuilt_smi, iso_flag=iso_flag)
        self.REBUILT_TYPE = get_type(rebuilt_smi)
        self.EXCLUDED_RING_SMILES = simplified_graph(excluded_smi, iso_flag=iso_flag)
        self.NODES = [node_one, node_two]
        self.REPR = " ".join(
            ["EDGE", self.NODES[0].SMILES, self.NODES[1].SMILES, self.get_label()]
        )
        self.HASH = hash(
            "||".join(
                ["EDGE", self.NODES[0].SMILES, self.NODES[1].SMILES, self.get_label()]
            )
        )

    def get_label(self):
        return "|".join(
            [
                self.EXCLUDE_TYPE,
                self.EXCLUDE_SMILES,
                self.EXCLUDED_RING_SMILES,
                self.REBUILT_TYPE,
                self.REBUILT_SMILES,
                self.REBUILT_RING_SMILES,
            ]
        )

    def __str__(self):
        return self.REPR


class Attr(object):
    """
    A Class of attributes
    """

    def __init__(self, smiles=None, property_list=None, input_str=None):
        if input_str:
            self.SMILES = input_str.split()[1]
            self.PROP_LIST = input_str.split()[2:]
        else:
            self.SMILES = smiles
            self.PROP_LIST = property_list
        if self.SMILES is None:
            raise ValueError("Att")

    def __str__(self):
        tot_list = ["ATTR", self.SMILES]
        tot_list.extend(self.PROP_LIST)
        return " ".join(tot_list)



from frag.network.decorate import decorate_smi
from frag.utils.vector_utils import get_max_ev_smi

if __name__ == "__main__":
    for item in [
        x.strip().split(",")
        for x in open(
            "/Users/abradley/fragalysis/frag/tests/data/enamine.csv"
        ).readlines()
    ]:
        print(item)
        smiles = decorate_smi(item[1])
        code = item[0]
        for smi in smiles:
            print(smi)
            print((get_max_ev_smi(smi)))



import unittest
from frag.alysis.cluster import DPMeans, KMeans


class ParserTest(unittest.TestCase):
    def test_dp_cluster(self):
        """
        Basic test to show clustering works.
        :return:
        """
        dp = DPMeans([(1.0, 1.0, 1.0), (3.0, 3.0, 3.0)], 1.0, 0, False)
        dp.run()
        self.assertEqual(len(dp.clusters), 2)

    def test_kmean_cluster(self):
        """
        Simple test to show clustering works.
        :return:
        """
        kmean = KMeans([(1.0, 1.0, 1.0), (3.0, 3.0, 3.0)], 2, 0, False)
        kmean.run()
        self.assertEqual(len(kmean.clusters), 2)



from .query import get_full_graph
from rdkit import Chem
import json

DATA_DICT = {}
TOT_SET = set()
GRAPH_URL = "neo4j-jun2018"


def iterate(input_smiles, counter, num_iterations):
    counter += 1
    if counter == num_iterations:
        return
    num_start = Chem.MolFromSmiles(input_smiles).GetNumHeavyAtoms()
    results = get_full_graph(input_smiles, graph_url=GRAPH_URL)
    for vector in results:
        for new_smiles in results[vector]:
            diff = Chem.MolFromSmiles(new_smiles).GetNumHeavyAtoms() - num_start
            if new_smiles not in TOT_SET:
                TOT_SET.add(new_smiles)
                DATA_DICT[counter].append((new_smiles, diff))
            iterate(new_smiles, counter, num_iterations)


def grow_smiles(smiles="CC(C)C(=O)Nc1cccc(C#N)c1", num_iterations=10):
    counter = -1
    for i in range(num_iterations):
        DATA_DICT[i] = []
    iterate(smiles, counter, num_iterations)



import os
import shutil


def do_for_dir(input_dir, num):
    os.chdir(input_dir)
    prop_dict = {
        "nodes.txt": [None, "smiles:ID(F2)", "hac:INT", "chac:INT", "osmiles"],
        "edges.txt": [None, ":START_ID(F2)", ":END_ID(F2)", "label"],
    }
    attr_smis = []
    attr_ids = []
    for x in open("attributes.txt").readlines():
        attr_smis.append(x.split()[1])
        attr_ids.append(x.split()[3])

    for f_name in prop_dict:
        out_f = open(f_name.replace(".txt", ".csv"), "w")
        for line in open(f_name).readlines():
            line_spl = line.split()
            out_l = []
            for i, x in enumerate(prop_dict[f_name]):
                if x:
                    out_l.append(line_spl[i])
            if f_name == "nodes.txt":
                if line_spl[1] in attr_smis:
                    cmpd_id = attr_ids[attr_smis.index(line_spl[1])]
                    out_l.append(cmpd_id)
                    # This is where we can add tags - like CHEAP - EXPENSIVE
                    out_l.append("Mol;MOL;F2")
                else:
                    out_l.append("")
                    out_l.append("F2")
                out_f.write(",".join(out_l) + "\n")
            elif f_name == "edges.txt":
                out_f.write(",".join(out_l) + "\n")
        out_f.flush()
        out_f.close()
    # Copy this back and add to final
    shutil.copy("edges.csv", "../edges_" + str(num) + ".csv")
    shutil.copy("nodes.csv", "../nodes_" + str(num) + ".csv")
    os.chdir("../")


node_list = ["nodes-header.csv"]
edge_list = ["edges-header.csv"]
for i in range(19):
    print(i)
    if i == 2:
        continue
    with open("edges-header.csv", "w") as out_f:
        out_f.write(",".join([x for x in [":START_ID(F2)", ":END_ID(F2)", "label"]]))
    with open("nodes-header.csv", "w") as out_f:
        out_f.write(
            ",".join(
                [
                    x
                    for x in [
                        "smiles:ID(F2)",
                        "hac:INT",
                        "chac:INT",
                        "osmiles",
                        "cmpd_id",
                        ":LABEL",
                    ]
                    if x
                ]
            )
            + "\n"
        )
    do_for_dir("ENA_SCREEN_" + str(i + 1), i + 1)
    node_list.append("nodes_" + str(i + 1) + ".csv")
    edge_list.append("edges_" + str(i + 1) + ".csv")
print(
    " ".join(
        [
            "/var/lib/neo4j/bin/neo4j-admin import",
            "--database",
            "new.db",
            "--nodes",
            '"' + ",".join(node_list) + '"',
            "--relationships:FRAG",
            '"' + ",".join(edge_list) + '"',
        ]
    )
)



import argparse
from tqdm import tqdm
from frag.utils.parser import get_file, parse_mols

if __name__ == "__main__":
    # Read in a SD or SMILES file - then write out into a specified directory
    parser = argparse.ArgumentParser(
        description="Convert a SMILES or SDFile to input for Astex Fragment network."
    )
    parser.add_argument("--input")
    parser.add_argument("--input_format", default="smi")
    parser.add_argument("--output")
    parser.add_argument("--output_format", default="smi")
    parser.add_argument("--chunk_size", default=None)
    parser.add_argument("--num_chunks", default=None)
    args = parser.parse_args()

    if args.chunk_size:
        chunk_size = int(args.chunk_size)
        num_chunks = None
    elif args.num_chunks:
        num_chunks = int(args.num_chunks)
        chunk_size = None

    mols = parse_mols(args.input, args.input_format)

    if num_chunks:
        out_files = [
            get_file(args.output, args.output_format, x) for x in range(num_chunks)
        ]
    counter = 0
    file_counter = 0
    for x in tqdm(mols):
        # Ignore None mols
        if x is None:
            continue
        if num_chunks:
            out_file = out_files[counter % num_chunks]
        else:
            if counter % chunk_size == 0:
                out_file = get_file(args.output, args.output_format, file_counter)
                file_counter += 1
        out_file.write(x)
        counter += 1



import argparse

if __name__ == "__main__":
    # Read in a SD or SMILES file - then write out into a specified directory
    parser = argparse.ArgumentParser(
        description="Convert a SMILES or SDFile to input for Astex Fragment network."
    )
    parser.add_argument("--input")
    parser.add_argument("--base_dir")
    args = parser.parse_args()
    # Now spawn up
    sdf_file_list = ["sdf_one.sdf", "sdf_two.sdf"]
    output_file_list = ["output_one", "output_two"]



import random
import logging

from frag.utils.network_utils import write_results, get_driver, canon_input

logger = logging.getLogger(__name__)


class ReturnObject(object):
    def __init__(
        self,
        start_smi,
        end_smi,
        label,
        edge_count,
        change_frag,
        iso_label,
        compound_ids,
    ):
        """
        Build this object.
        :param start_smi:
        :param end_smi:
        :param label:
        :param iso_label:
        :param frag_type:
        :param edge_count:
        :param change_frag:
        :param compound_ids: - list of compound_ids for the end_smi.
        """
        self.start_smi = start_smi
        self.end_smi = end_smi
        self.label = label
        self.iso_label = iso_label
        self.frag_type = None
        self.edge_count = edge_count
        self.change_frag = change_frag
        self.compound_ids = compound_ids

    def __str__(self):
        out_list = [self.label, str(self.edge_count), self.frag_type]
        return "_".join(out_list)


def find_double_edge_query(smiles):
    return (
        "MATCH (sta:F2 {smiles:'%(smiles)s'})-[nm:FRAG]-(mid:F2)-[ne:FRAG]-(end:Mol)"
        " where abs(sta.hac-end.hac) <= 3 and abs(sta.chac-end.chac) <= 1"
        " and sta.smiles <> end.smiles"
        " RETURN sta, nm, mid, ne, end"
        " order by split(nm.label, '|')[4],"
        " split(ne.label, '|')[2];" % {"smiles": smiles}
    )


def find_triple_edge_growth_query(
    smiles,
    heavy_atom_diff_min=6,
    heavy_atom_diff_max=10,
    mid_heavy_atom_diff_min=-1,
    mid_heavy_atom_diff_max=3,
):
    return (
        "MATCH (sta:F2 {smiles:'%(smiles)s'})-[nm:FRAG]-(mid_one:F2)-[ne:FRAG]-(mid:Mol)-[nm2:FRAG]-(mid_two:F2)-[ne2:FRAG]-(end:Mol)"
        " where end.hac-sta.hac > %(hacmin)d and end.hac-sta.hac <= %(hacmax)d"
        " and mid.hac-sta.hac > %(chacmin)d and mid.hac-sta.hac <= %(chacmax)d"
        " and sta.smiles <> mid.smiles and sta.smiles <> end.smiles "
        " WITH collect("
        "{"
        "end: end.smiles,"
        "mid: mid.smiles,"
        "frag_one: mid_one.smiles,"
        "frag_two: mid_two.smiles"
        "}"
        ") AS edges"
        " RETURN edges"
        % {
            "smiles": smiles,
            "hacmin": heavy_atom_diff_min,
            "hacmax": heavy_atom_diff_max,
            "chacmin": mid_heavy_atom_diff_min,
            "chacmax": mid_heavy_atom_diff_max,
        }
    )


def add_follow_ups_query(smiles):
    return (
        "MATCH (sta:F2 {smiles:'%(smiles)s'})-[nm:FRAG]-(mid:F2)-[ne:FRAG]-(end:Mol)"
        " where abs(sta.hac-end.hac) <= 3 and abs(sta.chac-end.chac) <= 1"
        " and sta.smiles <> end.smiles "
        " MERGE (end)-[:FOLLOW_UP]->(sta)" % {"smiles": smiles}
    )


def find_proximal_query(smiles):
    return (
        "match p = (n:F2{smiles:'%(smiles)s'})-[nm]-(m:Mol)"
        " where abs(n.hac-m.hac) <= 3 and abs(n.chac-m.chac) <= 1"
        " return n, nm, m"
        " order by split(nm.label, '|')[4];" % {"smiles": smiles}
    )


def get_type(r_group_form, sub_one, sub_two):
    if "." in r_group_form:
        if "C1" in sub_two:
            return "ring_linker"
        return "linker"
    if "C1" in sub_two:
        return "ring_replacement"
    return "replacement"


def define_double_edge_type(record):
    """
    Create return object from the graph query results returned for double edge jump molecules
    :param: query record
    :return: ReturnObject
    """
    mol_one = record["sta"]
    label = str(record["ne"]["label"].split("|")[4])
    iso_label = str(record["ne"]["label"].split("|")[5])
    change_frag = str(record["ne"]["label"].split("|")[2])
    mol_two = record["mid"]
    mol_three = record["end"]
    diff_one = mol_one["hac"] - mol_two["hac"]
    diff_two = mol_two["hac"] - mol_three["hac"]
    ret_obj = ReturnObject(
        mol_one["smiles"],
        mol_three["smiles"],
        label,
        2,
        change_frag,
        iso_label,
        mol_three["cmpd_ids"],
    )
    if "." in label:
        ret_obj.frag_type = "LINKER"
    elif diff_one >= 0 and diff_two >= 0:
        ret_obj.frag_type = "DELETION"
    elif diff_one <= 0 and diff_two <= 0:
        ret_obj.frag_type = "ADDITION"
    else:
        ret_obj.frag_type = "REPLACE"
    return ret_obj


def define_proximal_type(record):
    """
    Create return object from the graph query results returned for (naerby) proximal molecules
    :param: query record
    :return: ReturnObject
    """
    mol_one = record["n"]
    label = str(record["nm"]["label"].split("|")[4])
    iso_label = str(record["nm"]["label"].split("|")[5])
    change_frag = str(record["nm"]["label"].split("|")[2])
    mol_two = record["m"]
    ret_obj = ReturnObject(
        mol_one["smiles"],
        mol_two["smiles"],
        label,
        1,
        change_frag,
        iso_label,
        mol_two["cmpd_ids"],
    )
    if "." in label:
        ret_obj.frag_type = "LINKER"
    elif mol_one["hac"] - mol_two["hac"] > 0:
        ret_obj.frag_type = "DELETION"
    elif mol_one["hac"] - mol_two["hac"] < 0:
        ret_obj.frag_type = "ADDITION"
    else:
        ret_obj.frag_type = "REPLACE"
    return ret_obj


def organise(records, num_picks):
    """
    Create Output dictionary from the list of ReturnObject records
    :param: ReturnObject records
    :return: Output dictionary
    """
    out_d = {}
    smi_set = set()
    for rec in records:
        rec_key = str(rec)
        addition = {
            "change": rec.change_frag,
            "end": rec.end_smi,
            "compound_ids": rec.compound_ids,
        }
        if rec_key in out_d:
            out_d[rec_key]["addition"].append(addition)
        else:
            out_d[rec_key] = {"vector": rec.iso_label, "addition": [addition]}
        smi_set.add(rec.end_smi)
    if num_picks:
        max_per_hypothesis = num_picks / len(out_d)
    out_smi = []
    for rec in out_d:
        # TODO here is the logic as to ordering replacements
        if num_picks:
            random.shuffle(out_d[rec]["addition"])
            out_d[rec]["addition"] = out_d[rec]["addition"][:max_per_hypothesis]
        else:
            out_d[rec]["addition"] = out_d[rec]["addition"]
        out_smi.extend(out_d[rec])
    return out_d


def get_picks(smiles, num_picks, graph_url="neo4j", graph_auth="neo4j/neo4j"):
    smiles = canon_input(smiles)
    driver = get_driver(graph_url, graph_auth)
    with driver.session() as session:
        records = []
        for record in session.run(find_proximal_query(smiles)):
            ans = define_proximal_type(record)
            records.append(ans)
        for record in session.run(find_double_edge_query(smiles)):
            ans = define_double_edge_type(record)
            records.append(ans)
        for label in list(set([x.label for x in records])):
            # Linkers are meaningless
            if "." in label:
                continue
        if records:
            orga_dict = organise(records, num_picks)
            return orga_dict
        else:
            print("Nothing found for input: " + smiles)


def get_full_graph(
    smiles, graph_url="neo4j", graph_auth="neo4j/neo4j", isomericSmiles=True
):
    """
    For the given smiles and graph credentials, query the Neo4j database
    :param: smiles
    :param: graph_url - note that for local Neo4j graphs, this can be set to 'localhost'
    :param: graph_auth - username/password.
    :param: isomericSmiles True/False
    :return: Output dictionary - This is returned "as-is" from the Fragalysis-Backend api/graph query.
    """

    msg = f'get_full_graph("{smiles}", graph_url={graph_url}, isomericSmiles={isomericSmiles})'
    smiles = canon_input(smiles, isomericSmiles)
    msg += f' canon_smiles="{smiles}"'

    driver = get_driver(graph_url, graph_auth)
    records = []
    with driver.session() as session:
        debug_proximal_num_records = 0
        for record in session.run(find_proximal_query(smiles)):
            debug_proximal_num_records += 1
            ans = define_proximal_type(record)
            records.append(ans)
        msg += f" n_proximals={debug_proximal_num_records}"
        debug_double_edge_num_records = 0
        for record in session.run(find_double_edge_query(smiles)):
            debug_double_edge_num_records += 0
            ans = define_double_edge_type(record)
            records.append(ans)
        msg += f" n_double_edge={debug_double_edge_num_records}"
        for label in list(set([x.label for x in records])):
            # Linkers are meaningless
            if "." in label:
                continue

    if records:
        orga_dict = organise(records, None)
        orga_dict_len = len(orga_dict)

        msg += f" orga_dict_len={orga_dict_len}"
        logger.info(f"GRAPH -> {msg}")
        return orga_dict

    msg += f" (fall-through) ret=None"
    logger.info(f"GRAPH -> {msg}")
    return None


def custom_query(query, graph_url="neo4j", graph_auth="neo4j/neo4j"):
    driver = get_driver(graph_url, graph_auth)
    records = []
    with driver.session() as session:
        for record in session.run(query):
            records.append(record)
    return records


def write_picks(smiles, num_picks):
    img_dict = write_results(get_picks(smiles, num_picks))
    for key in img_dict:
        out_f = open(key + ".svg", "w")
        out_f.write(img_dict[key])



from frag.utils.rdkit_utils import (
    RDKitPh4,
    _get_c_of_mass,
    find_dist,
    _get_res_rmsds,
    get_res_atom_name,
    _get_res,
)
import unittest
from rdkit import Chem

SDF_DATA = """
     RDKit          3D

  8  8  0  0  0  0  0  0  0  0999 V2000
   -0.2375    1.2479   -0.2221 C   0  0  0  0  0  0  0  0  0  0  0  0
   -1.5033    1.0878    0.3249 C   0  0  0  0  0  0  0  0  0  0  0  0
   -1.9231   -0.1960    0.6598 C   0  0  0  0  0  0  0  0  0  0  0  0
   -1.4265   -1.1855   -0.1809 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.0741   -1.1112   -0.4792 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.5447    0.1297   -0.4205 C   0  0  0  0  0  0  0  0  0  0  0  0
    2.0371    0.2224   -0.4859 C   0  0  0  0  0  0  0  0  0  0  0  0
    2.5826   -0.1950    0.8038 N   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  2  0
  2  3  1  0
  3  4  2  0
  4  5  1  0
  5  6  2  0
  6  7  1  0
  7  8  1  0
  6  1  1  0
M  END

"""

PDB_DATA = """ATOM   1315  N   HIS A 174      29.516 -40.779  92.218  1.00 59.90           N
ATOM   1316  CA  HIS A 174      30.777 -40.579  92.954  1.00 55.77           C
ATOM   1317  C   HIS A 174      31.928 -40.122  92.037  1.00 48.90           C
ATOM   1318  O   HIS A 174      32.167 -40.710  90.977  1.00 53.82           O
ATOM   1319  CB  HIS A 174      31.169 -41.858  93.717  1.00 59.44           C
ATOM   1320  CG  HIS A 174      30.133 -42.338  94.691  1.00 63.61           C
ATOM   1321  ND1 HIS A 174      29.504 -41.501  95.589  1.00 63.97           N
ATOM   1322  CD2 HIS A 174      29.645 -43.580  94.931  1.00 66.30           C
ATOM   1323  CE1 HIS A 174      28.658 -42.202  96.324  1.00 66.58           C
ATOM   1324  NE2 HIS A 174      28.725 -43.467  95.945  1.00 69.74           N
"""


class Ph4Test(unittest.TestCase):
    def test_ph4_parser(self):
        """
        Test the pharmacophore generation.
        Need more test cases
        :return:
        """
        rdmol = Chem.MolFromMolBlock(SDF_DATA)
        rdkit_ph4 = RDKitPh4()
        feats = rdkit_ph4.generate_ph4_for_mol(rdmol=rdmol)
        self.assertEqual(len(feats), 5)
        self.assertLessEqual(
            [x[3] for x in feats],
            ["SingleAtomDonor", "BasicGroup", "Arom6", "ThreeWayAttach", "RH6_6"],
        )
        self.assertAlmostEqual([x[1] for x in feats][1], -0.195)


class CentreOfMassTest(unittest.TestCase):
    def test_centre_of_mass(self):
        rdmol = Chem.MolFromMolBlock(SDF_DATA)
        centre_of_mass = _get_c_of_mass(rdmol)
        self.assertAlmostEqual(centre_of_mass[0], -1.2499999999970868e-05)
        self.assertAlmostEqual(centre_of_mass[1], 1.2499999999995154e-05)
        self.assertAlmostEqual(centre_of_mass[2], -1.2499999999984746e-05)


class MathTest(unittest.TestCase):
    def test_square_dist(self):
        self.assertEqual(find_dist(8, 22, 12, 4, 6, 6), 308.0)


class ResTest(unittest.TestCase):
    def test_get_res_rmsd(self):
        input_data = [
            {
                "A": [1.0, 2.0, 3.0],
                "B": [1.0, 2.0, 3.0],
                "C": [1.0, 2.0, 3.0],
            },
            {
                "A": [2.0, 3.0, 4.0],
                "B": [1.0, 2.0, 3.0],
                "C": [4.0, 3.0, 2.0],
            },
            {
                "A": [-0.2375, 1.2479, -0.2221],
                "B": [-1.5033, 1.0878, 0.3249],
                "C": [-1.9231, -0.1960, 0.6598],
            },
        ]
        output_data = [
            [2.160246899469287, 3.8977444101257674],
            [2.160246899469287, 5.392922351255084],
            [3.8977444101257674, 5.392922351255084],
        ]
        test_output = _get_res_rmsds(input_data)
        print(test_output)
        for i, output in enumerate(test_output):
            self.assertEqual(len(output), 2)
            self.assertEqual(output_data[i][0], output[0])

    def test_get_res_name(self):
        pdb_mol = Chem.MolFromPDBBlock(PDB_DATA)
        conf = pdb_mol.GetConformer()
        atoms = pdb_mol.GetAtoms()
        for atom in atoms:
            atom
        unique_name, atom_name, position = get_res_atom_name(atom, conf)
        self.assertEqual(unique_name, "174_A_HIS")
        self.assertEqual(atom_name, "NE2")
        self.assertAlmostEqual(position[0], 28.725)
        self.assertAlmostEqual(position[1], -43.467)
        self.assertAlmostEqual(position[2], 95.945)

    def test_get_res(self):
        mol = Chem.MolFromPDBBlock(PDB_DATA)
        out_dict = _get_res(mol)
        self.assertListEqual(list(out_dict.keys()), ["174_A_HIS"])



from rdkit import Chem
from rdkit.Chem import AllChem
import numpy as np


def get_best_fit_plane(pts, weights=None):
    """

    :param pts:
    :param weights:
    :return:
    """
    linalg = np.linalg
    if weights is None:
        wSum = len(pts)
        origin = np.sum(pts, 0)
    origin /= wSum
    sums = np.zeros((3, 3), np.double)
    for pt in pts:
        dp = pt - origin
        for i in range(3):
            sums[i, i] += dp[i] * dp[i]
            for j in range(i + 1, 3):
                sums[i, j] += dp[i] * dp[j]
                sums[j, i] += dp[i] * dp[j]
    sums /= wSum
    vals, vects = linalg.eigh(sums)
    order = np.argsort(vals)
    normal = vects[:, order[0]]
    plane = np.zeros((4,), np.double)
    plane[:3] = normal
    plane[3] = -1 * normal.dot(origin)
    return plane


def get_pbf(plane, pts):
    denom = np.dot(plane[:3], plane[:3])
    denom = denom**0.5
    # add up the distance from the plane for each point:
    res = 0.0
    for pt in pts:
        res += np.abs(pt.dot(plane[:3]) + plane[3])
        res /= denom
        res /= len(pts)
    return res


def plane_best_fit_exit_vector(scaffold, repl_smarts, confId):
    """
    Get the exit vector for the best fit plane and the orthogonal plane.
    :param scaffold:
    :param repl_smarts:
    :param confId:
    :return:
    """
    # Get PBF plane for murcko scaffold only
    conf = scaffold.GetConformer(confId)
    if not conf.Is3D():
        print("This mol is not 3D - all PBFev angles will be 0 degrees")
        return [0]
    pts = np.array(
        [
            list(conf.GetAtomPosition(i))  # Get atom coordinates if not Xenon
            for i in range(scaffold.GetNumAtoms())
            if scaffold.GetAtomWithIdx(i).GetSymbol() != repl_smarts
        ]
    )
    # Plane is xyz vector with a c intercept adjustment
    plane = get_best_fit_plane(pts)
    pbf = get_pbf(plane, pts)
    # Where [#0] matches exit vector SMILES [*]
    patt = Chem.MolFromSmarts("[" + repl_smarts + "]-[*]")
    matches = scaffold.GetSubstructMatches(patt)
    if len(matches) == 0:
        return None

    # Calculate angles between exit vectors and the murcko plane of best fit
    exitVectors = {}
    denom = np.dot(plane[:3], plane[:3])
    denom = denom**0.5
    for n, match in enumerate(matches):
        out_isotope = scaffold.GetAtomWithIdx(match[0]).GetIsotope()
        evCoords = conf.GetAtomPosition(match[0])
        anchorCoords = conf.GetAtomPosition(match[1])
        v = np.array(
            (
                (evCoords[0] - anchorCoords[0]),
                (evCoords[1] - anchorCoords[1]),
                (evCoords[2] - anchorCoords[2]),
            )
        )
        angle = np.arcsin((np.dot(v, plane[:3])) / ((denom) * ((np.dot(v, v)) ** 0.5)))
        angle = np.abs(np.degrees(angle))
        exitVectors[out_isotope] = angle
    exitVectors["pbf"] = pbf
    return exitVectors


def get_exit_vector_for_xe_smi(input_smi, repl_smarts="At"):
    """
    Get the exit vector angles for a SMILES with Xen
    :param input_smi:
    :return:
    """
    mol = Chem.MolFromSmiles(input_smi.replace("Xe", repl_smarts))
    conf_id = AllChem.EmbedMolecule(mol)
    AllChem.UFFOptimizeMolecule(mol, confId=conf_id)
    return plane_best_fit_exit_vector(mol, repl_smarts, conf_id)


def get_max_ev_smi(smi):
    """
    Try three different size groups - and take the maximum.
    This is a fairly huge assumption. The variability in the vector coud also be a parameter
    :param smi:
    :return:
    """
    # Big group
    big = get_exit_vector_for_xe_smi(smi.replace("At", "Xe"), "At")
    # Mediume group
    medium = get_exit_vector_for_xe_smi(smi.replace("At", "Xe"), "K")
    # Little group
    small = get_exit_vector_for_xe_smi(smi.replace("At", "Xe"), "Li")
    out_d = {}
    for key in big:
        out_d[key] = max([big[key], medium[key], small[key]])
    return out_d



#!/usr/bin/env python

"""
A file splitter.

A split-like utility given an input file, an output basename and the number
of lines required in each split output file. Here the input file is expected
to have a one-line header, which is replicated in each output file.

Alan Christie
July 2018
"""

import argparse


def header_slit(input_file, output_base, output_size, extension=".smi"):
    """Splits the lines in an input file (including a header)
    into a series of output files.

    :param input_file: The input file
    :type input_file: ``str``
    :param output_base: The basename of the output files. The files are written
                        to the current working directory with a numerical
                        suffix and .smi. If the base is `x` the first output
                        file will be named `x_1[extension]`.
    :type output_base: ``str``
    :param output_size: The number of lines in each output file.
    :type output_size: ``int``
    :param extension: The extension for the output files.
    :type extension: ``str``
    """

    with open(input_file) as smiles_file:
        # Get the input file's header
        smiles_file.seek(0, 0)
        header = smiles_file.readline()

        file_line_count = 0
        file_number = 1
        output_file = None
        line = smiles_file.readline()
        while line and line.strip():
            if file_line_count == 0:
                # Start a new file and write the header
                name = output_base + "_" + str(file_number) + extension
                output_file = open(name, "w")
                output_file.write(header)
                file_line_count = 0

            output_file.write(line)
            file_line_count += 1

            if file_line_count == output_size:
                # Close file
                output_file.close()
                output_file = None
                file_line_count = 0
                file_number += 1

            # Next line...
            line = smiles_file.readline()

        # Done last input line.
        # Do we have anything un-written?
        if output_file:
            output_file.close()


def main():
    PARSER = argparse.ArgumentParser(
        description="File splitter."
        " Given a file this utility"
        " splits the lines"
        " replicating its header"
        " to new files."
    )
    PARSER.add_argument(
        "-i", "--input_file", help="The name of the input file.", required=True
    )
    PARSER.add_argument(
        "-o",
        "--output_base",
        help="The basename you want to use for your output"
        " files. Each output file will use this as a base"
        " name and will append a unique decimal number to"
        " the end before adding a .smi extension.",
        required=True,
    )
    PARSER.add_argument(
        "-s",
        "--output_size",
        help="The (maximum) number of lines in each output" " file.",
        required=True,
        type=int,
    )
    ARGS = PARSER.parse_args()

    header_slit(ARGS.input_file, ARGS.output_base, int(ARGS.output_size))


if __name__ == "__main__":
    main()



import argparse
import os

from frag.utils.network_utils import get_driver, add_attr, add_edge, add_node
from tqdm import tqdm


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Upload Nodes, Edges and Attributes")
    parser.add_argument("--base_dir")
    args = parser.parse_args()
    driver = get_driver()

    with driver.session() as session:
        for line in tqdm(open(os.path.join(args.base_dir, "nodes.txt")).readlines()):
            session.write_transaction(
                add_node,
                line.split()[1],
                line.split()[2],
                line.split()[3],
                line.split()[4],
            )
        for line in tqdm(open(os.path.join(args.base_dir, "edges.txt")).readlines()):
            session.write_transaction(
                add_edge, line.split()[1], line.split()[2], line.split()[3]
            )
        for line in tqdm(
            open(os.path.join(args.base_dir, "attributes.txt")).readlines()
        ):
            session.write_transaction(add_attr, line.split()[1], line.split()[3])



import os
import math
from rdkit.Chem import ChemicalFeatures
from rdkit import Chem


# Generate ph4s for a molecule
class RDKitPh4(object):
    factory = None

    def __init__(self):
        # Generate the factory on init
        self.get_factory()

    def get_factory(self):
        """
        Generate the Ph4 feature factory
        :return:
        """
        if self.factory is None:
            this_dir, this_filename = os.path.split(__file__)
            data_path = os.path.join(this_dir, "data", "RDKitPh4.fdef")
            self.factory = ChemicalFeatures.BuildFeatureFactory(data_path)
        return self.factory

    def generate_ph4_for_mol(self, rdmol):
        """
        Generate a pharmacophore from an input molecule and a feature factory.
        :param rdmol: the input RDKit molecule
        :param factory: the feature factory
        :return: a list of 4 tuples (x,y,z, feature)
        """
        feats = self.get_factory().GetFeaturesForMol(rdmol)
        return [
            (feat.GetPos().x, feat.GetPos().y, feat.GetPos().z, feat.GetType())
            for feat in feats
        ]


class RDKitAtom(object):
    def generate_atoms_for_mol(self, rdmol):
        """
        Generate the atoms from an input molecule and a feature factory.
        :param rdmol: the input RDKit molecule
        :return: a list of 4 tuples (x,y,z, atom description)
        """
        out_list = []
        conf = rdmol.GetConformer()
        for atom in rdmol.GetAtoms():
            atom_desc = self.get_atom_description(atom)
            atom_pos = conf.GetAtomPosition(atom.GetIdx())
            out_list.append((atom_pos.x, atom_pos.y, atom_pos.z, atom_desc))
        return out_list

    def get_atom_description(self, atom):
        """
        Generate a unique description of an atom
        :param atom: the input atom
        :return: a hash string of atomic number and  hybridization state
        """
        return "_".join(
            [str(x) for x in [atom.GetAtomicNum(), atom.GetHybridization()]]
        )


def _parse_ligand_sdf(input_file):
    """
    Function to parse a series of ligands - return RDKit mols.
    :param input_file: the file to parse
    :param input_type: the type of ligands
    :return: the molecules parsed
    """
    mols = Chem.SDMolSupplier(input_file)
    return mols


def _get_c_of_mass(rdmol):
    """
    Get the unweighted centre of mass of an RDKit Molecule
    :param rdmol:
    :return:
    """
    atoms = rdmol.GetAtoms()
    conf = rdmol.GetConformer()
    x_coord = y_coord = z_coord = 0.0
    numatoms = 0.0
    # Assume all heavy atoms have the same mass
    for atom in atoms:
        if atom.GetAtomicNum() == 1 or atom.GetSmarts() == "[*]":
            continue
        numatoms += 1.0
        coords = conf.GetAtomPosition(atom.GetIdx())

        x_coord += float(coords.x)
        y_coord += float(coords.y)
        z_coord += float(coords.z)
    # Now we have all the coords -> we want to loop through
    if numatoms == 0:
        raise ValueError("No atoms in Molecules")
    return x_coord / numatoms, y_coord / numatoms, z_coord / numatoms


def _get_waters(input_lines):
    """
    Very strict but efficient way of getting waters
    :param input_lines: the lines of a PDB file we want to consider
    :return:
    """
    return [x for x in input_lines if x[17:20] == "HOH"]


def _get_water_coords(waters):
    """Helper function to get the coordinates from a load of waters."""
    rd_waters = Chem.MolFromPDBBlock("\n".join(waters))
    out_list = []
    if rd_waters is None:
        print("Warning - unable to parse waters.")
    if rd_waters is not None:
        # Check the waters exist
        conf = rd_waters.GetConformer()
        # Delete them for this protein
        for i in range(rd_waters.GetNumAtoms()):
            cp = conf.GetAtomPosition(i)
            if rd_waters.GetAtomWithIdx(i).GetSmarts() != "O":
                print("Warning - skipping a water")
                continue
            out_list.append((cp.x, cp.y, cp.z))
    return out_list


def _get_file(file_path, output_format, file_counter):
    if output_format == "smi":
        return Chem.SmilesWriter(file_path + "_" + str(file_counter) + ".smi")
    else:
        return Chem.SDWriter(file_path + "_" + str(file_counter) + ".sdf")


def _parse_mols(input_file, input_format):
    if input_format == "smi":
        return Chem.SmilesMolSupplier(input_file, delimiter=",")
    else:
        return Chem.SDMolSupplier(input_file)


def _parse_pdb(data):
    return Chem.MolFromPDBFile(data)


def find_dist(mol_1_x, mol_1_y, mol_1_z, mol_2_x, mol_2_y, mol_2_z):
    """Function to find the square distance between two points in 3D
    Takes two len=3 tuples
    Returns a float"""
    return (
        pow((mol_1_x - mol_2_x), 2)
        + pow((mol_1_y - mol_2_y), 2)
        + pow((mol_1_z - mol_2_z), 2)
    )


def _get_mean_dist(res_one, res_two):
    tot_dist = 0.0
    num_matches = 0
    for atom in res_one:
        if atom in res_two:
            # Find the distance
            dist = find_dist(
                res_one[atom][0],
                res_one[atom][1],
                res_one[atom][2],
                res_two[atom][0],
                res_two[atom][1],
                res_two[atom][2],
            )
            tot_dist += dist
            num_matches += 1
    # Find the mean square distance
    return float(tot_dist) / float(num_matches)


def _get_res_rmsds(input_res_list):
    """
    Helper function to get the RMSDs for a list of Residues.
    :param input_res_list: The list of RDKit molecules of residues
    :return: a list of lists of RMSDS.
    """
    # Calculate RMSD from corresponding
    num_res = len(input_res_list)
    tot_res_rmsd_list = []
    for i in range(num_res):
        this_res_rmsd_list = []
        for j in range(num_res):
            if i == j:
                continue
            mean_dist = _get_mean_dist(input_res_list[i], input_res_list[j])
            # Append the root mean square distance
            root_mean_sqr = math.sqrt(mean_dist)
            this_res_rmsd_list.append(root_mean_sqr)
        tot_res_rmsd_list.append(this_res_rmsd_list)
    return tot_res_rmsd_list


def get_res_atom_name(atom, conf):
    """
    Get the information for each atom
    :param atom: the input atom
    :param conf: the molecular conformer
    :return: the unqiue residue level name, the atom name and the position
    """
    res_info = atom.GetPDBResidueInfo()
    atom_pos = conf.GetAtomPosition(atom.GetIdx())
    position = [atom_pos.x, atom_pos.y, atom_pos.z]
    identifiers = [
        res_info.GetResidueNumber(),
        res_info.GetChainId(),
        res_info.GetResidueName(),
        res_info.GetAltLoc(),
    ]
    unique_name = "_".join([str(x) for x in identifiers if x != " "])
    atom_name = res_info.GetName().strip()
    return unique_name, atom_name, position


def _get_res(mol):
    """
    Get a list of residues with RDKit mols (RDMol,RES_NAME)
    :param input_data:
    :return:
    """
    out_dict = {}
    conf = mol.GetConformer()
    atoms = mol.GetAtoms()
    for atom in atoms:
        # Get res_name
        res_name, atom_name, position = get_res_atom_name(atom, conf)
        if res_name in out_dict:
            out_dict[res_name][atom_name] = position
        else:
            out_dict[res_name] = {atom_name: position}
    return out_dict
